@@ -1,20 +1,11 @@
 #ifndef BITS_H
 #define BITS_H
 
-/* default lib includes */
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
-
-#include "macros.h"
-
-#include <limits.h>
-#include <stddef.h>
-#include <stdint.h>
-
 #define UINT64_SUM_OVERFLOWS(a, b) \
 	(a > (uint64_t)-1 - b)
 
+#define BIT(n) (1u << (n))
+
 size_t nearest_power(size_t num) ATTR_CONST;
 
 unsigned int bits_required8(uint8_t num) ATTR_CONST;
@@ -23,7 +14,7 @@ static inline
 unsigned int bits_required16(uint16_t num)
 {
 	return (num <= 0xff) ? bits_required8(num)
-		: 8 + bits_required16(num >> 8);
+		: 8 + bits_required8(num >> 8);
 }
 static inline
 unsigned int bits_required32(uint32_t num)
